'use strict';

var path = require('path');
var fs = require('fs');
var util = require('util');

function NotiChannel(config) {
  const { id, providers, strategy } = config;
  const strategicSend = strategy(id);
  const send = async (params) => {
    const respnose = await strategicSend(
      params,
      typeof providers === "function" ? await providers(params) : providers
    );
    return {
      ...respnose,
      channelId: id
    };
  };
  const channel = {
    id,
    send
  };
  return channel;
}

class NotiError extends Error {
  code;
  metadata;
  errors;
  constructor(code, message, options) {
    super(message, options);
    this.name = this.constructor.name;
    this.code = code;
    this.metadata = options?.metadata ?? {};
    this.errors = options?.errors;
  }
}
const getCreateNotiError = (cls) => {
  return (err, code, options) => {
    if (err instanceof cls) {
      return err;
    }
    return new cls(
      code,
      err instanceof Error ? err.message : `Unexpected ${cls.name}`,
      { cause: err, ...options }
    );
  };
};
class NotiChannelError extends NotiError {
  static create = getCreateNotiError(NotiChannelError);
}
class NotiProviderError extends NotiError {
  static create = getCreateNotiError(NotiProviderError);
}
class NotiTemplateError extends NotiError {
  static create = getCreateNotiError(NotiTemplateError);
}

async function sendWithFallbackStrategy(params, channelId, providers, errors = []) {
  const [provider, ...remainingProviders] = providers;
  if (typeof provider === "undefined") {
    throw new NotiChannelError(
      "channel_send_failure",
      "Failed to send with fallback strategy",
      { errors, metadata: { channelId } }
    );
  }
  try {
    const response = await provider.send(params);
    return {
      ...response,
      providerId: provider.id
    };
  } catch (err) {
    errors.push(
      NotiProviderError.create(err, "provider_send_failure", {
        metadata: { channelId, providerId: provider.id }
      })
    );
    return await sendWithFallbackStrategy(
      params,
      channelId,
      remainingProviders,
      errors
    );
  }
}
function fallbackStrategy(channelId) {
  return async (params, providers) => {
    return await sendWithFallbackStrategy(params, channelId, providers);
  };
}

function randomStrategy(channelId) {
  return async (params, providers) => {
    const index = Math.floor(Math.random() * providers.length);
    const provider = providers[index];
    if (typeof provider === "undefined") {
      throw new NotiChannelError(
        "channel_send_failure",
        "Failed to pick provider with random strategy",
        { metadata: { channelId } }
      );
    }
    try {
      const response = await provider.send(params);
      return {
        ...response,
        providerId: provider.id
      };
    } catch (err) {
      throw new NotiChannelError(
        "channel_send_failure",
        "Failed to send with random strategy",
        {
          errors: [
            NotiProviderError.create(err, "provider_send_failure", {
              metadata: { channelId, providerId: provider.id }
            })
          ],
          metadata: { channelId }
        }
      );
    }
  };
}

const readFile = util.promisify(fs.readFile);
const renderRaw = async (templatePath) => {
  return await readFile(templatePath, { encoding: "utf8" });
};

const dirLevels = { CHANNEL: 0, TOPIC: 1, PARAM: 2 };
function getTemplateMapKey({
  channel,
  topic,
  param
}) {
  return `${channel}--${topic}--${param}`;
}
function getTemplateMap(dirPath, dirLevel = 0, channel, topic) {
  const templateMap = /* @__PURE__ */ new Map();
  if (dirLevel > dirLevels.PARAM) {
    return templateMap;
  }
  const dirents = fs.readdirSync(dirPath, { withFileTypes: true });
  for (const dirent of dirents) {
    const currentName = dirent.name;
    const currentPath = path.join(dirPath, currentName);
    if (dirLevel === dirLevels.PARAM) {
      const param = currentName.split(".")[0];
      const templateIdentifier = { channel, topic, param };
      templateMap.set(
        getTemplateMapKey(templateIdentifier),
        currentPath
      );
      continue;
    }
    if (dirent.isDirectory()) {
      const map = getTemplateMap(
        currentPath,
        dirLevel + 1,
        dirLevel === dirLevels.CHANNEL ? currentName : channel,
        dirLevel === dirLevels.TOPIC ? currentName : topic
      );
      for (const [key, value] of map.entries()) {
        templateMap.set(key, value);
      }
    }
  }
  return templateMap;
}

const isPlainObject = (obj) => {
  return Object.prototype.toString.call(obj) === "[object Object]";
};
const defaults = (object, defaultObject) => {
  const result = object;
  for (const key in result) {
    if (isPlainObject(result[key]) && isPlainObject(defaultObject[key])) {
      result[key] = defaults(result[key], defaultObject[key]);
    }
  }
  for (const key in defaultObject) {
    if (typeof result[key] === "undefined") {
      result[key] = defaultObject[key];
    }
  }
  return result;
};

const defaultConfig = {
  path: path.resolve("templates"),
  data: {},
  options: {},
  renderer: {
    "": renderRaw
  }
};
function NotiTemplate(config) {
  const templateConfig = defaults(config, defaultConfig);
  const templateMap = getTemplateMap(templateConfig.path);
  const render = async (identifier, data = {}, options = {}) => {
    const templateKey = getTemplateMapKey(identifier);
    try {
      const templatePath = templateMap.get(templateKey);
      if (typeof templatePath === "undefined") {
        throw new NotiTemplateError(
          "template_not_found",
          `Template not found '${templateKey}'`,
          { metadata: { templateId: templateKey } }
        );
      }
      const templateFilename = path.basename(templatePath);
      const templateType = templateFilename.split(".").slice(1).join(".");
      const templateRenderer = templateConfig.renderer[templateType];
      if (typeof templateRenderer !== "function") {
        throw new NotiTemplateError(
          "template_renderer_not_found",
          `Renderer not found '${templateType}' for template '${templateKey}'`,
          { metadata: { templateId: templateKey, rendererId: templateType } }
        );
      }
      const templateData = defaults(data, templateConfig.data);
      const templateRendererOptions = defaults(options, templateConfig.options);
      return await templateRenderer(
        templatePath,
        templateData,
        templateRendererOptions
      );
    } catch (err) {
      throw NotiTemplateError.create(err, "template_render_failure", {
        metadata: { templateId: templateKey }
      });
    }
  };
  return {
    templateMap,
    render
  };
}

exports.NotiChannel = NotiChannel;
exports.NotiChannelError = NotiChannelError;
exports.NotiError = NotiError;
exports.NotiProviderError = NotiProviderError;
exports.NotiTemplate = NotiTemplate;
exports.NotiTemplateError = NotiTemplateError;
exports.fallbackStrategy = fallbackStrategy;
exports.getTemplateMap = getTemplateMap;
exports.getTemplateMapKey = getTemplateMapKey;
exports.randomStrategy = randomStrategy;
exports.renderRaw = renderRaw;
